<?php
/**
 * OSB V2-Payment Module version 1.2.0 for osCommerce 2.3.x. Support contact : support@osb.pf.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author    Lyra Network (http://www.lyra-network.com/)
 * @copyright 2014-2017 Lyra Network and contributors
 * @license   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html  GNU General Public License (GPL v2)
 * @category  payment
 * @package   osb
 */

/**
 * This file is an access point for the OSB payment gateway to validate an order.
 */

if (key_exists('vads_hash', $_POST) && isset($_POST['vads_hash']) && key_exists('vads_result', $_POST) && isset($_POST['vads_result'])) {
    // restore session if this is an IPN call.

    $osCsid = substr($_POST['vads_order_info'], strlen('session_id='));
    $_POST['osCsid'] = $osCsid;
    $_GET['osCsid'] = $osCsid;

    // for cookie based sessions ...
    $_COOKIE['osCsid'] = $osCsid;
    $_COOKIE['cookie_test'] = 'please_accept_for_session';

    require_once 'checkout_process.php';
} else {
    require_once('includes/application_top.php');

    global $osb_response, $language, $messageStack;

    $paymentMethod = str_replace('payment_method=', '', $_REQUEST['vads_order_info2']);

    switch ($paymentMethod) {
        case 'osb':
            require_once (DIR_FS_CATALOG . 'includes/modules/payment/osb.php');
            $paymentObject = new osb();
            break;

        case 'osb_multi':
            require_once (DIR_FS_CATALOG . 'includes/modules/payment/osb_multi.php');
            $paymentObject = new osb_multi();
            break;

        case 'osb_choozeo':
            require_once (DIR_FS_CATALOG . 'includes/modules/payment/osb_choozeo.php');
            $paymentObject = new osb_choozeo();
            break;

        default:
            require_once (DIR_FS_CATALOG . "includes/languages/$language/modules/payment/osb.php");
            $messageStack->add_session('header', MODULE_PAYMENT_OSB_TECHNICAL_ERROR, 'error');

            tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true));
            break;
    }

    require_once (DIR_FS_CATALOG . 'includes/classes/osb_response.php');
    $osb_response = new OsbResponse(
            $_REQUEST,
            constant($paymentObject->prefix . 'CTX_MODE'),
            constant($paymentObject->prefix . 'KEY_TEST'),
            constant($paymentObject->prefix . 'KEY_PROD')
    );

    $from_server = $osb_response->get('hash') != null;

    // check authenticity
    if (! $osb_response->isAuthentified()) {
        $messageStack->add_session('header', MODULE_PAYMENT_OSB_TECHNICAL_ERROR, 'error');

        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL', true));
    }

    if ($paymentObject->_is_order_paid()) {
        // messages to display on payment result page
        if (constant($paymentObject->prefix . 'CTX_MODE') == 'TEST') {
            $messageStack->add_session('header', MODULE_PAYMENT_OSB_GOING_INTO_PROD_INFO . '<a href="https://secure.osb.pf/html/faq/prod" target="_blank">https://secure.osb.pf/html/faq/prod</a>', 'success');
        }

        tep_redirect(tep_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL', true));
    } else {
        $return_mode = '_' . ($from_server ? 'POST' : constant($paymentObject->prefix . 'RETURN_MODE'));

        if ($return_mode == '_POST') {
            $action = tep_href_link(FILENAME_CHECKOUT_PROCESS, '', 'SSL', true);
            $fields = '';

            foreach ($$return_mode as $key => $value) {
                $fields .= '<input type="hidden" name="' . $key . '" value="' . htmlentities($value, ENT_QUOTES, 'UTF-8') . '" />' . "\n";
            }

            echo <<<EOT
                <html>
                    <body>
                        <form action="$action" method="POST" name="checkout_process_osb_form">
                            $fields
                        </form>

                        <script type="text/javascript">
                            window.onload = function() {
                                document.checkout_process_osb_form.submit();
                            };
                        </script>
                    </body>
                </html>
EOT;
        } else {
            tep_redirect(tep_href_link(FILENAME_CHECKOUT_PROCESS, http_build_query($$return_mode), 'SSL', true));
        }
    }
}
